#!/usr/bin/env python3
"""NCBI functions."""

import ujson
from tolkein import tofetch
from tolkein import tolog

LOGGER = tolog.logger(__name__)

GBIF_API = "https://www.gbif.org/api"


def fetch_gbif_taxlist(params):
    """Fetch gbif taxon entries."""
    limit = params.get("limit", 5)
    offset = params.get("offset", 0)
    highertaxon_key = params.get("root", None)
    url = (
        "%s/species/search?advanced=false&rank=SPECIES&rank=SUBSPECIES&status=ACCEPTED&status=DOUBTFUL"
        % GBIF_API
    )
    if highertaxon_key is not None:
        url += "&highertaxon_key=%s" % highertaxon_key
    url += "&limit=%d&offset=%d" % (limit, offset)
    page = tofetch.fetch_url(url)
    data = ujson.decode(page)
    return data


def stream_gbif_taxa(root=None):
    """Stream gbif taxon entries."""
    params = {"root": root, "limit": 50, "offset": 0}
    data = {"results": [], "endOfRecords": False}
    while not data["endOfRecords"]:
        data = fetch_gbif_taxlist(params)
        for entry in data.get("results", []):
            yield (entry["key"], entry)
        params["offset"] += params["limit"]


def prepare_xref_rows(identifiers, meta):
    """Convert identifiers to a set of rows for output."""
    rows = []
    common = {"species": meta["species"]}
    if "family" in meta:
        common.update({"family": meta["family"]})
    if "subspecies" in meta:
        common.update({"subspecies": meta["subspecies"]})
    if "NCBI" in identifiers:
        common.update({"ncbiTaxonId": identifiers["NCBI"]["id"]})
    else:
        common.update({"gbifTaxonId": "GBIF:%s" % identifiers["GBIF"]["id"]})
    for db, entry in identifiers.items():
        row = {**common}
        row.update({"xref": "%s:%s" % (db, str(entry["id"]))})
        row.update({"sourceUrl": entry["url"]})
        row.update({"source": entry["source"]})
        row.update({"sourceSlug": entry["id"]})
        row.update({"sourceStub": entry["url"].replace(str(entry["id"]), "")})
        rows.append(row)
    return rows


def fetch_gbif_identifiers(taxon, *, xrefs=None):
    """Fetch gbif identifiers for a taxon."""
    if xrefs is None:
        xrefs = []
    xrefs = set(["NCBI"] + xrefs)
    url = "%s/wikidata/species/%s?locale=en" % (GBIF_API, str(taxon))
    page = tofetch.fetch_url(url)
    identifiers = {
        "GBIF": {
            "id": taxon,
            "url": "https://www.gbif.org/species/%s" % taxon,
            "source": "GBIF taxonKey",
        }
    }
    if page is not None:
        data = ujson.decode(page)
        for entry in data.get("identifiers", []):
            label = entry.get("label", {}).get("value", "None")
            db = label.split(" ")[0]
            if db in xrefs:
                identifiers.update(
                    {db: {"id": entry["id"], "url": entry["url"], "source": label}}
                )
    return identifiers


def gbif_parser(_params, opts, *args, **kwargs):
    """Parse GBIF taxa and identifiers."""
    parsed = []
    for root in opts["gbif-root"]:
        for key, meta in stream_gbif_taxa(root):
            identifiers = fetch_gbif_identifiers(key, xrefs=opts["gbif-xref"])
            parsed += prepare_xref_rows(identifiers, meta)
    return parsed



#!/usr/bin/env python3

"""Analysis methods."""

from tolkein import tolog

from .hub import index_templator

LOGGER = tolog.logger(__name__)


def index_template(taxonomy_name, opts):
    """Index template (includes name, mapping and types)."""
    parts = ["analysis", taxonomy_name, opts["hub-name"], opts["hub-version"]]
    template = index_templator(parts, opts)
    return template



#!/usr/bin/env python3

"""
Parse a local or remote data source.

Usage:
    genomehubs parse [--btk] [--btk-root STRING...]
                     [--wikidata PATH] [--wikidata-root STRING...] [--wikidata-xref STRING...]
                     [--gbif] [--gbif-root STRING...] [--gbif-xref STRING...]
                     [--ncbi-datasets-genome PATH] [--outfile PATH]
                     [--refseq-mitochondria] [--refseq-organelles]
                     [--refseq-plastids] [--refseq-root NAME]
                     [-h|--help] [-v|--version]

Options:
    --btk                        Parse assemblies in BlobToolKit
    --btk-root STRING            Scientific name of root taxon
    --gbif                       Parse taxa in GBIF
    --gbif-root STRING           GBIF taxon ID of root taxon
    --gbif-xref STRING           Include link to external reference from GBIF (e.g. NBN, BOLD)
    --meta PATH                  YAML format file of analysis metadata
    --wikidata PATH              Parse taxa in WikiData dump
    --wikidata-root STRING       WikiData taxon ID of root taxon
    --wikidata-xref STRING       Include link to external reference from WikiData (e.g. NBN, BOLD)
    --ncbi-datasets-genome PATH  Parse NCBI Datasets genome directory
    --outfile PATH               Save parsed output to file
    --refseq-mitochondria        Parse mitochondrial genomes from the NCBI RefSeq
                                 organelle collection
    --refseq-organelles          Parse all genomes from the NCBI RefSeq organelle
                                 collection
    --refseq-plastids            Parse plastid genomes from the NCBI RefSeq organelle
                                 collection
    --refseq-root NAME           Name (not taxId) of root taxon
    -h, --help                   Show this
    -v, --version                Show version number
"""

import re
import sys
from pathlib import Path

from docopt import docopt
from tolkein import tofile
from tolkein import tolog

from .btk import btk_parser
from .config import config
from .hub import load_types
from .hub import order_parsed_fields

# from .ncbi import ncbi_datasets_summary_parser
from .ncbi import ncbi_genome_parser
from .ncbi import refseq_organelle_parser
from .version import __version__
from .wikidata import wikidata_parser

LOGGER = tolog.logger(__name__)

PARSERS = {
    "btk": {"func": btk_parser, "params": None, "types": "btk"},
    "ncbi-datasets-genome": {
        "func": ncbi_genome_parser,
        "params": None,
        "types": "assembly",
    },
    # "ncbi-datasets-summary": {
    #     "func": ncbi_datasets_summary_parser,
    #     "params": None,
    #     "types": "assembly",
    # },
    "refseq-mitochondria": {
        "func": refseq_organelle_parser,
        "params": ("mitochondrion"),
        "types": "organelle",
    },
    "refseq-organelles": {
        "func": refseq_organelle_parser,
        "params": ("mitochondrion", "plastid"),
        "types": "organelle",
    },
    "refseq-plastids": {
        "func": refseq_organelle_parser,
        "params": ("plastid"),
        "types": "organelle",
    },
    "wikidata": {"func": wikidata_parser, "params": None, "types": "wikidata"},
}


def main(args):
    """Parse data sources."""
    options = config("parse", **args)

    for option in options["parse"]:
        if option in PARSERS:
            params = PARSERS[option]["params"]
            if params is None:
                params = options["parse"][option]
            LOGGER.info("Parsing %s" % option)
            types = load_types(PARSERS[option]["types"])
            names = load_types(PARSERS[option]["types"], part="names")
            parsed = PARSERS[option]["func"](
                params, options["parse"], types=types, names=names
            )
            files = []
            if isinstance(parsed, tuple):
                parsed, files = parsed
            data = order_parsed_fields(parsed, types, names)
            tofile.write_file(options["parse"]["outfile"], data)
            filepath = Path(options["parse"]["outfile"])
            outdir = filepath.parent
            suff = re.compile(r"\.[^\.]+$")
            if filepath.name.endswith(".gz"):
                stem = re.sub(suff, "", filepath.stem)
            else:
                stem = filepath.stem
            if types:
                types["file"]["name"] = filepath.name
                tofile.write_file("%s/%s.types.yaml" % (outdir, stem), types)
            if names:
                names["file"]["name"] = filepath.name
                tofile.write_file("%s/%s.names.yaml" % (outdir, stem), names)
            if files:
                tofile.write_file("%s/%s.files.yaml" % (outdir, stem), files)


def cli():
    """Entry point."""
    if len(sys.argv) == sys.argv.index("parse") + 1:
        args = docopt(__doc__, argv=[])
    else:
        args = docopt(__doc__, version=__version__)
    main(args)


if __name__ == "__main__":
    cli()
